#!/bin/bash
#
# mount-manager: bin/api/config-json-api.sh
#
# Require:
#   mount-manager-api.sh
#   bash-common-api:message-api.sh
#   bash-common-api:json-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t is_null_or_empty )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function is_null_or_empty not found.' >&2
  exit 1
fi

declare -r __JSON_CONFIG_FILE_V1_DEFAULT__=~/.config/mount-manager.json
declare -r __JSON_CONFIG_FILE_V2_DEFAULT__=~/.config/cclaude.rocks/mount-manager.json
declare -gx __JSON_CONFIG__=
declare -gx __JSON_CONFIG_FILE__=

declare -i CHECKLIST_HEIGHT=0
declare -i CHECKLIST_WIDTH=0
declare -i CHECKLIST_LISTHEIGHT=0

if [ -z "${MESSAGE_API_HIDE_DATE:-}" ] ; then
  declare -gx MESSAGE_API_HIDE_DATE='true'
fi

#
# API: config_json_init
#
function config_json_init {
  local -r use_cache="$1"

  echoinfo 'Loading configuration'

  local __config_file__=
  __config_file__="$( config_json_get_config_file )" || return $? # config_json_init

  if [ ! -f "${__config_file__}" ] ; then
    config_json_try_configuration_file_migration || return $?
  fi
  if [ ! -f "${__config_file__}" ] ; then
    echofatal "Configuration file is missing: '${__config_file__}'"
    return 101
  fi

  __JSON_CONFIG__="$( __config_json_compress__ "${__config_file__}" )" || return $?
  __JSON_CONFIG__="$( samba_flattern_config <<<"${__JSON_CONFIG__}" )" || return $?
  __JSON_CONFIG__="$( sshfs_flattern_config <<<"${__JSON_CONFIG__}" )" || return $?

  if [ "${use_cache}" != 'true' ] ; then
    __JSON_CONFIG_FILE__="$( mktemp --suffix=mount-manager )" || return $?
    declare -grx __JSON_CONFIG_FILE__ # Store temp filename

    (
      [ "${VERBOSE:-}" = 'true' ] && set -x
      jq '.' <<<"${__JSON_CONFIG__}" >"${__JSON_CONFIG_FILE__}"
    ) || return $?

    if [ "${CONFIG_JSON_KEEP_TEMP_FILES:-}" != 'true' ] ; then
      trap 'rm -f -- "${__JSON_CONFIG_FILE__}"' EXIT
    fi
    __JSON_CONFIG__= # Do not keep configuration in memory
  fi
  declare -grx __JSON_CONFIG__
}

function config_json_try_configuration_file_migration {
  if [ -f "${__JSON_CONFIG_FILE_V1_DEFAULT__}" ] ; then
    echowarn "Change configuration file location"

    local configdirv2=
    configdirv2="$( basename "${__JSON_CONFIG_FILE_V2_DEFAULT__}" )" || return $?
    local -r configdirv2

    if [ ! -d "${configdirv2}" ] ; then
      mkdir -vp "${configdirv2}" || return $?
    fi

    mv -v "${__JSON_CONFIG_FILE_V1_DEFAULT__}" "${__JSON_CONFIG_FILE_V2_DEFAULT__}" || return $?
  fi
}

function __config_json_compress__ {
  local -r __config_file__="$1"

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c 'walk(if type == "object" then with_entries(select(.key | test("@") | not)) else . end)' "${__config_file__}" || return $?
  )
}

function DISABLED__config_json_compress__ {
  local -r __config_file__="$1"

  local keys_to_delete=
  keys_to_delete="$(
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -r '.mount | keys | to_entries[] | select( .value | contains("@")) | .value' "${__config_file__}"
    )" || return $?

  local __config__=
  __config__="$(
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c '.' "${__config_file__}"
    )" || return $?

  local key_to_delete=
  while read -r key_to_delete ; do
    if [ -n "${key_to_delete}" ] ; then
      __config__="$(
      [ "${VERBOSE:-}" = 'true' ] && set -x
      jq -c --arg key "${key_to_delete}" 'del( .mount | .[$key] )' <<<"${__config__}"
      )" || return $?
    fi
  done <<<"${keys_to_delete}"
  echo "${__config__}"
}

function __config_json_run_jq__ { # was: __json_run_jq__
  if [ -n "${__JSON_CONFIG__}" ] ; then
    (
      [ "${VERBOSE:-}" = 'true' ] && set -x
      jq -c "$@" <<<"${__JSON_CONFIG__}"
    ) || return $?
  else
    local __config_file__=
    if [ -n "${__JSON_CONFIG_FILE__}" ] ; then
      __config_file__="${__JSON_CONFIG_FILE__}"
    else
      __config_file__="$( config_json_get_config_file )" || return $? # config_json_init
    fi
    (
      [ "${VERBOSE:-}" = 'true' ] && set -x
      jq -c "$@" "${__config_file__}"
    ) || return $?
  fi
}

function __config_json_get_option_value__ { # was: __json_get_option_value__
  local -r varname="$1"

  # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
  __config_json_run_jq__ --arg 'name' "${varname}" '.options | .[ $name ]' || return $?
}

function __config_json_get_valid_value__ { # was: __json_get_valid_value__
  local -r value="$1"
  local -r default_value="$2"

  if [ -z "${value}" ]; then
    echo "${default_value}"
  elif [ "${value}" = "null" ]; then
    echo "${default_value}"
  else
    echo "${value}"
  fi
}

#
# API: config_json_get_config_file
#
function config_json_get_config_file { # was: json_get_config_file
  echo "${__JSON_CONFIG_FILE_V2_DEFAULT__}"
}

function __config_json_compute_size__ { # was: __compute_size__
  local -r json_param_name="$1"
  local -r computed_value="$2"
  local -r default_value="$3"

  local size=
  size="$( __config_json_get_valid_value__ "$( __config_json_get_option_value__ "${json_param_name}" )" '' )"

  if [ -z "${size}" ] ; then
    size="${computed_value}"
  fi

  if [ "${size}" = "0" ] ; then
    echo "${default_value}"
  else
    echo "${size}"
  fi
}

#
# API: config_json_init_CHECKLIST_values
#
function config_json_init_CHECKLIST_values { # was: init_CHECKLIST_values
  local size=
  local tty_checklist_height=0
  local tty_checklist_width=0

  if size="$( stty size 2>/dev/null )" ; then
    tty_checklist_height="$( cut -d ' ' -f1 <<<"${size}" )"
    tty_checklist_width="$( cut -d ' ' -f2 <<<"${size}" )"
  fi

  CHECKLIST_HEIGHT="$( __config_json_compute_size__ 'CHECKLIST_HEIGHT' "${tty_checklist_height}" 20 )"
  CHECKLIST_WIDTH="$(  __config_json_compute_size__ 'CHECKLIST_WIDTH'  "${tty_checklist_width}"  80 )"

  local json_checklist_listheight=
  json_checklist_listheight="$( __config_json_get_valid_value__ "$( __config_json_get_option_value__ "CHECKLIST_LISTHEIGHT" )" '' )"
  if [ -z "${json_checklist_listheight}" ] ; then
    CHECKLIST_LISTHEIGHT="$(( CHECKLIST_HEIGHT - 7 ))"
  else
    CHECKLIST_LISTHEIGHT="${json_checklist_listheight}"
  fi

  echodebug "CHECKLIST_HEIGHT=${CHECKLIST_HEIGHT}"
  echodebug "CHECKLIST_WIDTH=${CHECKLIST_WIDTH}"
  echodebug "CHECKLIST_LISTHEIGHT=${CHECKLIST_LISTHEIGHT}"
}

#
# API: config_json_check_configuration
#
function config_json_check_configuration { # was: check_configuration
  echoinfo 'Checking configuration'

  # Check JSON
  local -i json_rc=0
  __config_json_run_jq__ -r '.[]' >/dev/null || json_rc="$?"
  local -ri json_rc

  if [ "${json_rc}" -ne 0 ]; then
    echofatal "Bad JSON '$( config_json_get_config_file )' : ${json_rc}"
    return "${json_rc}"
  fi

  # __config_json_check_type_ids__ 'diskimage' || return $? # No ID
  __config_json_check_type_ids__ 'iscsi' || return $?
  __config_json_check_type_ids__ 'nfs' || return $?
  __config_json_check_type_ids__ 'samba' || return $?

  # TODO more checks
  __config_json_check_structure__ || return $?
  __config_json_check_content_samba__ || return $?

  echoinfo 'Checking configuration - done'
}

function __config_json_check_structure__ { # was: _check_structure_
  local type_entries=
  type_entries="$( __config_json_check_structure_get_all_type_entries__ )" || return $?

  declare -ga __check_structure_ids__=()
  local type_entry=

  while read -r type_entry ; do
    if [ -n "${type_entry}" ] ; then
      __config_json_check_structure_for_entry__ "${type_entry}" || return $?
    fi
  done <<<"${type_entries}"

  __check_structure_ids__=() # No more needed
}

function __config_json_check_entry_display_warn_if_null_or_empty__ { # was: _check_entry_display_warn_if_null_or_empty_
  local -r id="$1"
  local -r jq_filter="$2"
  local -r json="$3"

  local value=
  value="$( jq -r "${jq_filter}" <<<"${json}" )" || return $?

  if is_null_or_empty "${value}" ; then
    echowarn "Wrong '${jq_filter}' for id='${id}'"
    jq '.' <<<"${json}" >&2
  fi
}

function __config_json_check_entry_samba__ { # was: _check_entry_samba_
  local -r _entry_="$1"

  local id=
  id="$( jq -r '.id' <<<"${_entry_}" )" || return $?
  local -r id

  if is_null_or_empty "${id}" ; then
    echofail "Wrong id='${id}' for: ${_entry_}"
    return 100
  fi

  __config_json_check_entry_display_warn_if_null_or_empty__ "${id}" '.ip' "${_entry_}" || return $?
  __config_json_check_entry_display_warn_if_null_or_empty__ "${id}" '.username' "${_entry_}" || return $?
}

function __config_json_check_content_samba__ { # was: _check_content_samba_
  local _entries_=
  _entries_="$( __config_json_run_jq__ -c '.mount.samba // [] | .[]' )" || return $?
  local -r _entries_

  while read -r _entry_ ; do
    if [ -n "${_entry_}" ] ; then
      __config_json_check_entry_samba__ "${_entry_}" || return $?
    fi
  done <<<"${_entries_}"
}

function __config_json_check_structure_for_entry__ { # was: __check_structure_for_entry__
  local -r type_entry="$1"

  local id=
  id="$( jq '.id' <<<"${type_entry}" )" || return $?

  if is_null_or_empty "${id}" ; then
    echofatal "id not valid: '${id}' (null or empty) for type '${type_entry}'"
    jq '.' <<<"${type_entry}" >&2
    return 100
  fi

  if [ -n "$( __config_json_check_structure_is_id_defined__ )" ] ; then
    echofatal "id already define: '${id}'"
    jq '.' <<<"${type_entry}" >&2
    return 100
  fi
  __check_structure_ids__+=("${id}")

  local mount_all=
  mount_all="$( jq '.["mount-all"]' <<<"${type_entry}" )" || return $?

  if ! is_null_or_empty "${mount_all}" ; then
    if ! is_boolean "${mount_all}" ; then
      echofatal "mount-all not valid: '${mount_all}' - Must be true or false"
      jq '.' <<<"${type_entry}" >&2
      return 100
    fi
  fi

  local mount_default=
  mount_default="$( jq '.["mount-default"]' <<<"${type_entry}" )" || return $?

  if ! is_null_or_empty "${mount_default}" ; then
    if ! is_boolean "${mount_default}" ; then
      echofatal "mount-default not valid: '${mount_default}' - Must be true or false"
      jq '.' <<<"${type_entry}" >&2
      return 100
    fi
  fi
}

function __config_json_check_structure_is_id_defined__ { # was: __check_structure_is_id_defined__
  local current=

  for current in "${__check_structure_ids__[@]}" ; do
    grep "^${id}$" <<<"${current}"
  done
}

function __config_json_check_structure_get_all_type_entries__ { # was: __check_structure_get_all_type_entries__
  local _types_=
  _types_="$( __config_json_run_jq__ -r '.mount | keys[]' )" || return $?

  local _type_=
  while read -r _type_ ; do
    if [ -n "${_type_}" ] ; then
      # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
      __config_json_run_jq__ -c --arg 'type' "${_type_}" '.mount | .[ $type ][]' || return $?
    fi
  done <<<"${_types_}"
}

function __config_json_check_type_ids__ { # was: _check_type_ids_
  local -r type="$1"

  local ids=
  # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
  ids="$(
    __config_json_run_jq__ -r --arg 'type' "${type}" '.mount | .[ $type ] // [] | .[].id'
    )" || return $?

  local error='false'
  local id=

  while read -r id ; do
    if [ -n "${id}" ] ; then
      if ! __config_json_is_id_valid__ "${id}" ; then
        echofatal "Illegal ID found: '${id}' - Check your configuration for type: '${type}'"
        error='true'
      fi
    fi
  done <<<"${ids}"

  if [ "${error}" = 'false' ] ; then
    return 0
  else
    return 1
  fi
}

function __config_json_is_id_valid__ { # was: __is_id_valid__
  local -r id="$1"

  if [ -z "${id}" ] ; then
    return 1
  elif [ "${id}" = 'null' ] ; then
    return 1
  elif [[ "${id}" != "${id//[[:blank:]]/}" ]] ; then
    return 1
  else
    return 0
  fi
}

#
# API: config_json_get_type_ids
#
function config_json_get_type_ids { # was: get_type_ids
  local -r type="$1"

  local filter=
  # shellcheck disable=SC2016 # Expressions don't expand in single quotes, use double quotes for that.
  if [ "${PARAM_ALL}" = 'true' ] ; then
    filter='.mount | .[ $type ] // [] | .[] | select( (.["mount-all"]==true) or (.["mount-default"]==true) ) | .id'
  elif [ "${PARAM_DEFAULT}" = 'true' ] ; then
    filter='.mount | .[ $type ] // [] | .[] | select( .["mount-default"]==true ) | .id'
  else
    filter='.mount | .[ $type ] // [] | .[].id'
  fi

  echodebug "PARAM_ALL: '${PARAM_ALL}'"
  echodebug "PARAM_DEFAULT: '${PARAM_DEFAULT}'"
  echodebug "filter: '${filter}'"

  local _ids_=
  _ids_="$( __config_json_run_jq__ -r --arg 'type' "${type}" "${filter}" )" || return $?

  if [ -n "${_ids_}" ] ; then
    local id=
    while read -r id ; do
      if __config_json_is_id_valid__ "${id}" ; then
        echo "${id}"
      else
        echofatal "Bad ID found: '${id}' for '${type}' - Check your configuration"
      fi
    done <<<"${_ids_}"
  fi
}

#
# API: config_json_get_type_value type id valuename
#
function config_json_get_type_value { # was: get_type_value
  local -r type="$1"
  local -r id="$2"
  local -r valuename="$3"

  if [ -z "${type}" ] || [ -z "${id}" ] || [ -z "${valuename}" ] ; then
    echofatal "${FUNCNAME[0]}( '${type}' '${id}' '${valuename}' ) - Internal error"
    echofatal "${FUNCNAME[*]}"
    return 100
  fi

  local result_type=
  local result=
  if [ "${valuename}" = "${valuename%[]}[]" ] ; then
    # An array is expected
    local -r array="${valuename%[]}"

    result_type="$(
      # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
      __config_json_run_jq__ -r \
        --arg 'type' "${type}" \
        --arg 'id'   "${id}" \
        --arg 'name' "${array}" \
        '.mount | .[ $type ] // [] | .[] | select( .id == $id ) | .[ $name ] | type'
      )" || return $?
    if [ "${result_type}" != 'array' ] ; then
      echofatal "${FUNCNAME[0]}( '${type}' '${id}' '${valuename}' ) expect an array, found '${result_type}'"
      echofatal "Context: ${FUNCNAME[*]}"
      return 10
    fi

    result="$(
      # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
      __config_json_run_jq__ -r \
        --arg 'type' "${type}" \
        --arg 'id'   "${id}" \
        --arg 'name' "${array}" \
        '.mount | .[ $type ] // [] | .[] | select( .id == $id ) | .[ $name ] // [] | .[]'
      )" || return $?
  else
    # A simple value is expected (not an array)
    result_type="$(
      # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
      __config_json_run_jq__ -r \
        --arg 'type' "${type}" \
        --arg 'id'   "${id}" \
        --arg 'name' "${valuename}" \
        '.mount | .[ $type ] // [] | .[] | select( .id == $id ) | .[ $name ] | type'
      )" || return $?
    if [ "${result_type}" != 'string' ] ; then
      echofatal "${FUNCNAME[0]}( '${type}' '${id}' '${valuename}' ) expect a string, found '${result_type}'"
      return 10
    fi

    result="$(
      # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes
      __config_json_run_jq__ -r \
        --arg 'type' "${type}" \
        --arg 'id'   "${id}" \
        --arg 'name' "${valuename}" \
        '.mount | .[ $type ] // [] | .[] | select( .id == $id ) | .[ $name ] // empty'
      )" || return $?
  fi
  if [ -z "${result}" ] ; then
    echowarn "${FUNCNAME[0]}( '${type}' '${id}' '${valuename}' ) give empty result."
  fi
  echo "${result}"
}

#
# API: config_json_get_object [JQ_OPTIONS] JQ_FILTER
#
function config_json_get_object { # was: get_object
  __config_json_run_jq__ "$@" || return $?
}

#
# API: config_json_get_password_by_type_id TYPE ID
#
function config_json_get_password_by_type_id { # was: get_password_by_type_id
  local -r type="$1"
  local -r id="$2"

  local gpg_pass=
  gpg_pass="$( config_json_get_type_value "${type}" "${id}" 'gpg' )" || return $?

  local gpg_test=
  gpg_test="$( __config_json_get_valid_value__ "${gpg_pass}" 'NOTFOUND' )" || return $?

  if [ "${gpg_test}" = 'NOTFOUND' ]; then
    config_json_get_type_value "${type}" "${id}" 'password' || return $?
  else
    decrypt_from_json "${gpg_pass}" || return $?
  fi
}
