#!/bin/bash
#
# mount-manager:bin/api/iscsi-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi

declare -gr ISCSI_SCRIPT="${MOUNT_MANAGER_PATH}/mount-manager-iscsi.sh"

# Internal
function __init_ISCSI_VERBOSE_string__ {
  if [ "${VERBOSE}" = 'true' ] ; then
    declare -g ISCSI_VERBOSE='--verbose'
  else
    declare -g ISCSI_VERBOSE=
  fi
}

# Internal
function __iscsi_get_portal_list__ {
  local selected_ids=
  selected_ids="$( get_all_iscsi_ids )" || return $?
  local -r selected_ids

  (
    local selected_id=
    while read -r selected_id ; do
      if [ -n "${selected_id}" ] ; then
        get_iscsi_portal_by_mountname "${selected_id}" || return $?
      fi
    done <<<"${selected_ids}"
  ) | sort | uniq
}

##
## API: build_checkboxes_for_iscsi
##
function build_checkboxes_for_iscsi {
  echoinfo "${FUNCNAME[0]}()"

  local iscsi_id=
  local mountdir=
  local mounted_status=

  for iscsi_id in $( get_all_iscsi_ids ); do
    mountdir="$( get_iscsi_mountdir "${iscsi_id}" )"

    update_checkboxes "${mountdir}" "${iscsi_id}" || return $?
  done
}

##
## API: get_iscsi_mountdir ISCSI_ID
##
function get_iscsi_mountdir {
  get_mountdir "$1" || return $?
}

##
## API: iscsi_logout
##
function iscsi_logout {
  __init_ISCSI_VERBOSE_string__ || return $?

  local portal_list=
  portal_list="$( __iscsi_get_portal_list__ )" || return $?
  local -r portal_list

  local portal=
  while read -r portal ; do
    if [ -n "${portal}" ] ; then
      echoinfo "iscsi logout: ${portal}"
      echodebug "sudo bash ${ISCSI_SCRIPT} ${ISCSI_VERBOSE} --portal \"${portal}\" --logout"
      (
        set -x
        # shellcheck disable=SC2086
        sudo bash "${ISCSI_SCRIPT}" ${ISCSI_VERBOSE} --portal "${portal}" --logout
      ) || true # Ignore errors
    fi
  done <<<"${portal_list}"
}

##
## API: mount_all_iscsi
##
function mount_all_iscsi {
  local selected_ids=
  selected_ids="$( get_all_iscsi_ids )" || return $?
  local -r selected_ids

  local selected_id=
  while read -r selected_id ; do
    if [ -n "${selected_id}" ] ; then
      mount_iscsi_by_id "${selected_id}" || return $?
    fi
  done <<<"${selected_ids}"
}

##
## API: mount_iscsi_by_id ID
##
function mount_iscsi_by_id {
  local -r mountname="$1"

  local mountdir=
  mountdir="$( get_iscsi_mountdir "${mountname}" )" || return $?

  mount_iscsi_by_mountdir "${mountdir}" || return $?
}

##
## API: mount_iscsi_by_mountdir MOUNTDIR
##
function mount_iscsi_by_mountdir {
  local -r mountdir="$1"

  local mountname=
  mountname="$( basename "${mountdir}" )" || return $?

  echodebug "mount_iscsi_by_mountdir '${mountdir}' - mountname='${mountname}'"

  local mounted_status=
  mounted_status="$( get_mounted_status "${mountdir}" )" || return $?
  if [ "${mounted_status}" = 'on' ] ; then
    echowarn "Path '${mountdir}' already mounted."
    return 0
  fi

  local portal=
  portal="$( get_iscsi_portal_by_mountname "${mountname}" )" || return $?
  if [ -z "${portal}" ]; then
   echofatal "portal value not found for '${mountname}'"
   return 100
  fi

  local iqn=
  iqn="$( get_iscsi_iqn_by_mountname "${mountname}" )" || return $?

  local path=
  path="$( get_iscsi_path_by_mountname "${mountname}" )" || return $?

  __init_ISCSI_VERBOSE_string__ || return $?

  (
    [ "${VERBOSE}" = "true" ] && set -x
    # shellcheck disable=SC2086
    sudo bash "${ISCSI_SCRIPT}" ${ISCSI_VERBOSE} --mount --portal "${portal}" --path "${path}" --target "${mountdir}" --iqn "${iqn}" --real-user "${REALUSER}"
  ) || return $?
}

##
## API: umount_all_iscsi
##
function umount_all_iscsi {
  local selected_ids=
  selected_ids="$( get_all_iscsi_ids )" || return $?
  local -r selected_ids

  local selected_id=
  while read -r selected_id ; do
    if [ -n "${selected_id}" ] ; then
      umount_iscsi_by_id "${iscsi_id}" || true # Ignore errors on umount
    fi
  done <<<"${selected_ids}"
}

##
## API: umount_iscsi_by_id ISCSI_ID
##
function umount_iscsi_by_id {
  local -r mountname="$1"

  local mountdir=
  mountdir="$( get_iscsi_mountdir "${mountname}" )" || return $?

  echodebug "${FUNCNAME[0]}( '${mountname}' ) -> '${mountdir}'"
  umount_iscsi_by_mountdir "${mountdir}" || return $?
}

##
## API: umount_iscsi_by_mountdir MOUNTDIR
##
function umount_iscsi_by_mountdir {
  local -r mountdir="$1"

  if mount | grep "${mountdir}" --quiet; then
    echodebug "${FUNCNAME[0]}( '${mountdir}' )"

    local mountname=
    mountname="$( basename "${mountdir}" )"

    local portal=
    portal="$( get_iscsi_portal_by_mountname "${mountname}" )" || return $?

    if [ -z "${portal}" ]; then
      echofatal "portal value not found for '${mountname}'"
      return 100
    fi

    (
      [ "${VERBOSE}" = "true" ] && set -x
      sudo bash "${ISCSI_SCRIPT}" --portal "${portal}" --target "${mountdir}" --umount
    ) || return $?
  else
    echodebug "'${mountdir}' not mount"
  fi
}
