#!/bin/bash
#
# mount-manager:bin/api/mount-manager-api.sh
#
declare -a CHECKBOXES
declare -a MOUNTED
declare -a UMOUNTED

declare -ri MOUNT_MANAGER_SHARES_CHMOD='750'

# shellcheck disable=SC1091
source "${API_PATH}/mount-root-home-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/diskimage-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/iscsi-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/iscsi-config-json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/nfs-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/samba-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/sshfs-api.sh" || exit $?

#
# API: get_mountdir mountname
#
# mountname : directory relative name's (this is the id)
#
function get_mountdir {
  local -r mountname="$1"

  local mount_root_home=
  mount_root_home=$( get_mount_root_home ) || return $?

  echodebug "${FUNCNAME[0]}( '${mountname}' ) - > '${mount_root_home}/${mountname}'"
  echo "${mount_root_home}/${mountname}"
  return 0
}

#
# API: create_mountdir_if_needed mountdir
#
# mountdir:
#
function create_mountdir_if_needed {
  local -r mountdir="$1"

  echodebug "${FUNCNAME[0]}( (${mountdir}' )"

  if [ ! -d "${mountdir}" ]; then
    __mount_manager_prepare_share_root_for_user__ || return $?

    echoinfo "Create new share folder ${mountdir}"
    mkdir -pv "${mountdir}" || return $?
  else
    echodebug "${FUNCNAME[0]}( '${mountdir}' ) - Folder already exist."
  fi

  chmod "${MOUNT_MANAGER_SHARES_CHMOD}" "${mountdir}" || return $?
}

function __mount_manager_prepare_share_root_for_user__ {
  local mount_root_home=
  mount_root_home=$( get_mount_root_home ) || return $?
  local -r mount_root_home

  if [ ! -d "${mount_root_home}" ]; then
    echoinfo "Create new share folder '${mount_root_home}' (sudo access required)"

    echodebug "${FUNCNAME[0]}() : mount_root_home='${mount_root_home}'"
    (
      set -e
      sudo mkdir -pv "${mount_root_home}" || return $?
      sudo chown "${REALUSER}:${REALUSER}" "${mount_root_home}" || return $?
      sudo chmod "${MOUNT_MANAGER_SHARES_CHMOD}" "${mount_root_home}" || return $?
    ) || return $?
  fi
}

#
# API: umount_by_mountdir mountdir
#
# mountdir : Full path of mount folder
#
function umount_by_mountdir {
  local -r mountdir="$1"

  if mount | grep --quiet "${mountdir}" ; then
    local -i rc=0
    umount -v "${mountdir}" || rc=$?
    if [ "${rc}" -eq 32 ] ; then
      sudo umount -v "${mountdir}" || return $?
    else
      return "${rc}"
    fi
  else
    echodebug "'${mountdir}' not mount."
  fi
}

function get_mounted_status {
  local -r mountdir="$1"

  if mount | grep --quiet "${mountdir}" ; then
    echo "on"
  else
    echo "off"
  fi
  return 0
}

function ___update_context_according_to_mount_status__ {
  local -r mounted_status="$1"

  if [ "${mounted_status}" = 'on' ]; then
    MOUNTED+=("${mountdir}")
  else
    UMOUNTED+=("${mountdir}")
  fi
}

function update_checkboxes {
  local -r __mountdir__="$1"
  local -r __sharepath__="$2"

  local mounted_status=
  mounted_status="$( get_mounted_status "${__mountdir__}" )" || return $?

  ___update_context_according_to_mount_status__ "${mounted_status}" || return $?
  CHECKBOXES+=("${__mountdir__}" "${__sharepath__}" "${mounted_status}")
}

function compute_display_sizes {
  local size=
  size="$( stty size )" || return $?

  CHECKLIST_HEIGHT="$( cut -d ' ' -f1 <<<"${size}" )"
  CHECKLIST_WIDTH="$( cut -d ' ' -f2 <<<"${size}" )"
  CHECKLIST_LISTHEIGHT="$(( CHECKLIST_HEIGHT - 7 ))"
}

#
# API: select_mountdirs
#
function select_mountdirs {
  CHECKBOXES=()
  MOUNTED=()
  UMOUNTED=()

  build_checkboxes_for_sshfs || return $?
  build_checkboxes_for_samba || return $?
  build_checkboxes_for_nfs || return $?
  build_checkboxes_for_iscsi || return $?
  build_checkboxes_for_diskimage || return $?

  local to_mount_list=
  local whiptail_rc=

  if [ "${#CHECKBOXES[@]}" = 0 ]; then
    echofatal 'Configuration file is empty or is not a valid json file.'
    return 112
  fi

  compute_display_sizes || return $?

  to_mount_list=$(
    [ "${VERBOSE}" = "true" ] && set -x
    whiptail --separate-output \
    --title 'Personnal Mount Manager' \
    --checklist 'Please pick entries to mount' \
    "${CHECKLIST_HEIGHT}" "${CHECKLIST_WIDTH}" "${CHECKLIST_LISTHEIGHT}" \
    "${CHECKBOXES[@]}" 3>&1 1>&2 2>&3
    )
  whiptail_rc=$?

  echodebug "checklist whiptail_rc=${whiptail_rc}"

  if [ ${whiptail_rc} -eq 0 ]; then
    if [ ${#MOUNTED[@]} -ne 0 ]; then
      if [ -z "${to_mount_list}" ]; then

        # Cancel or umount all ?
        if ! whiptail --yesno "Would you like to umount all ?" --defaultno 20 60 2 ; then # not yes
          echofatal 'Umount all cancel by user.'
          return 10
        fi
      fi
    fi

    echodebug "to_mount_list=${to_mount_list}"
    [ "${#MOUNTED[@]}"  = 0 ] || echodebug "MOUNTED=${MOUNTED[*]}"
    [ "${#UMOUNTED[@]}" = 0 ] || echodebug "UMOUNTED=${UMOUNTED[*]}"
  else
    echofatal "Cancel by user. (${whiptail_rc})"
    return 11
  fi

  local status=
  local mountdir=
  local i=

  for ((i = 0; i < ${#UMOUNTED[@]}; i++)) ; do
    mountdir="${UMOUNTED[$i]}"

    if echo "${to_mount_list}" | grep --quiet "${mountdir}" ; then
      status='Mounting'

      mount_by_mountdir "${mountdir}" || return $?
    else
      status='Not mounted (no change)'
    fi

    echodebug "[${mountdir}] ${status}"
  done

  for ((i = 0; i < ${#MOUNTED[@]}; i++)) ; do
    mountdir="${MOUNTED[$i]}"

    if echo "${to_mount_list}" | grep --quiet "${mountdir}" ; then
      status='Already mounted (no change)'
    else
      status='Umounting'
      umount_by_mountdir "${mountdir}" || return $?
    fi

    echodebug "[${mountdir}] ${status}"
  done
}

function mount_all {
  mount_all_iscsi     || return $? # 1
  mount_all_nfs       || return $? # 2
  mount_all_samba     || return $? # 3
  mount_all_sshfs     || return $? # 4

  mount_all_diskimage || return $? # 5
}

function umount_all {
  umount_all_diskimage || true # 5 (reverse order)

  umount_all_sshfs     || true # 4
  umount_all_samba     || true # 3
  umount_all_nfs       || true # 2
  umount_all_iscsi     || true # 1
}

# Internal
function __mount_by_mountdir_get_type__ {
  local -r mountdir="$1"

  local configured_dir=
  local list=

  list="$( get_all_samba_mountdir )" || return $?
  while read -r configured_dir; do
    if [ "${mountdir}" = "${configured_dir}" ]; then
      echo 'samba'
      return 0
    fi
  done <<<"${list}"

  list="$( get_all_sshfs_mountdir )" || return $?
  while read -r configured_dir; do
    if [ "${mountdir}" = "${configured_dir}" ]; then
      echo 'sshfs'
      return 0
    fi
  done <<<"${list}"

  local id=
  list="$( get_all_diskimage_ids )" || return $?
  while read -r id; do
    if [ -n "${id}" ] ; then
      computed_dir="$( get_diskimage_mountdir "${id}" )"

      if [ "${mountdir}" = "${computed_dir}" ]; then
        echo 'diskimage'
        return 0
      fi
    fi
  done <<<"${list}"

  list="$( get_all_iscsi_ids )" || return $?
  while read -r id; do
    if [ -n "${id}" ] ; then
      computed_dir="$( get_iscsi_mountdir "${id}" )"

      if [ "${mountdir}" = "${computed_dir}" ]; then
        echo 'iscsi'
        return 0
      fi
    fi
  done <<<"${list}"

  echofatal "${FUNCNAME[0]}( '${mountdir}' ) - Can't find id."
  return 50
}

# Internal
function mount_by_mountdir {
  local -r mountdir="$1"

  local type=
  type="$( __mount_by_mountdir_get_type__ "${mountdir}" )" || return $?

  case "${type}" in
    'diskimage')
      mount_diskimage_by_mountdir "${mountdir}" || return $?
      ;;

    'iscsi')
      mount_iscsi_by_mountdir "${mountdir}" || return $?
      ;;

    'samba')
      mount_samba_by_mountdir "${mountdir}" || return $?
      ;;

    'sshfs')
      mount_sshfs_by_mountdir "${mountdir}" || return $?
      ;;

    *)
      echofatal "${FUNCNAME[0]}( '${mountdir}' ) - Internal error: type=${type}"
      return 1
      ;;
  esac
}

function __list_id__ {
  get_all_iscsi_ids     || return $?
  get_all_nfs_ids       || return $?
  get_all_samba_ids     || return $?
  get_all_sshfs_ids     || return $?

  get_all_diskimage_ids || return $?
}

function list_id {
  local list=
  list="$( __list_id__ )" || return $?

  sort <<<"${list}"
}

function __list_mountdir__ {
  get_all_iscsi_mountdir     || return $?
  get_all_nfs_mountdir       || return $?
  get_all_samba_mountdir     || return $?
  get_all_sshfs_mountdir     || return $?

  get_all_diskimage_mountdir || return $?
}

function list_mountdir {
  local list=
  list="$( __list_mountdir__ )" || return $?

  sort <<<"${list}"
}

function mountdirs {
  local mountdir=

  while [ "$#" -gt 0 ] ; do
    mountdir="$1"
    shift

    mount_by_mountdir "${mountdir}" || return $?
  done
}

function umountdirs {
  local mountdir=

  while [ "$#" -gt 0 ] ; do
    mountdir="$1"
    shift

    umount_by_mountdir "${mountdir}" || true # Ignore error on umount
  done
}
