#!/bin/bash
#
# mount-manager:bin/api/samba-api.sh
#

# Internal
function __build_checkboxes_for_samba_for_id__ {
  local -r id="$1"

  local json_object=
  json_object="$(
    # shellcheck disable=SC2016 # (info): Expressions don't expand in single quotes, use double quotes for that.
    config_json_get_object -c \
      --arg 'id'   "${id}" \
      '.mount | .[ "samba-flattern" ] // [] | .[] | select( .id == $id )'
    )" || return $?
  local -r json_object

  local mountdir=
  mountdir="$( jq -r '.["mount-point"] // empty' <<<"${json_object}" )" || return $?
  if [ -z "${mountdir}" ] ; then
    echofatal "${FUNCNAME[0]}( '${id}' '${json_object}' )"' - .["mount-point"] not found.'
    return 100
  fi
  local -r mountdir

  local share_name=
  share_name="$( jq -r '.["source-share"] // empty' <<<"${json_object}" )" || return $?
  if [ -z "${share_name}" ] ; then
    echofatal "${FUNCNAME[0]}( '${id}' '${json_object}' )"' - .["source-share"] not found.'
    return 100
  fi
  local -r share_name

  local ip=
  ip="$( jq -r '.ip // empty' <<<"${json_object}" )" || return $?
  if [ -z "${ip}" ] ; then
    echofatal "${FUNCNAME[0]}( '${id}' '${json_object}' ) - .ip not found."
    return 100
  fi
  local -r ip

  local share_path=
  share_path="$( __build_samba_sharepath__ "${ip}" "${share_name}" )" || return $?
  local -r share_path

  update_checkboxes "${mountdir}" "${share_path}" || return $?
}

# Internal
function __build_samba_sharepath__ {
  local -r server_ip="$1"
  local -r server_share="$2"

  echo "//${server_ip}/${server_share}"
}

# Internal
function __mount_samba__ {
  local -r username="$1"
  local -r password="$2"
  local -r server_ip="$3"
  local -r server_share="$4"
  local -r samba_id="$5" # Also server_prefix

  echodebug "${FUNCNAME[0]}() \"${username}\" \"*******\" \"${server_ip}\" \"${server_share}\" \"${samba_id}\""

  if [ -z "${server_ip}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing 'server_ip' value."
    return 100
  fi
  if [ -z "${server_share}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing 'server_share' value."
    return 100
  fi
  if [ -z "${samba_id}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing 'samba_id' value."
    return 100
  fi

  local mountdir=
  mountdir="$( get_samba_mountdir "${samba_id}" "${server_share}" )"

  local mounted_status=
  mounted_status="$( get_mounted_status "${mountdir}" )" || return $?
  if [ "${mounted_status}" = 'on' ] ; then
    echowarn "Path '${mountdir}' already mounted."
    return 0
  fi

  local sharepath=
  sharepath="$( __build_samba_sharepath__ "${server_ip}" "${server_share}" )"

  local user_id=
  user_id="$( id --user )"

  local group_id=
  group_id="$( id --group )"

  create_mountdir_if_needed "${mountdir}" || return $?

  local options="username=${username},password=${password},gid=${group_id},uid=${user_id}"
  local optional=

  #
  # https://linux.die.net/man/8/mount.cifs
  #
  optional="$( get_samba_optional_options "${samba_id}" )" || return $?
  echodebug "samba_id '${samba_id}' - optional '${optional}'"

  if [ "${optional}" = "null" ] || [ -z "${optional}" ] ; then
    options+=',iocharset=utf8'
    options+=',sec=ntlmssp'
    options+=',vers=2.0'

    # CIFS VFS: Autodisabling the use of server inode numbers.
    # This server doesn't seem to support them properly. Hardlinks will not be recognized on this mount.
    # Consider mounting with the "noserverino" option to silence this message.
    options+=',noserverino'
  else
    options+=",${optional}"
  fi

  local -i mountrc=0
  (
    [ "${VERBOSE}" = "true" ] && set -x
    sudo mount -v -t cifs "${sharepath}" -o "${options}" "${mountdir}"
  ) || mountrc=$?

  if [ "${mountrc}" -ne 0 ] ; then
    echoerror "${FUNCNAME[0]}() - Error for id='${samba_id}': smb://${username}:*******@${server_ip}/${server_share}/"
    # FIXME must return an error (required to not try to mount already mounted path)
  fi
}

# Internal
function __mount_samba_from_json_entry__ {
  local -r entry_json="$1"

  if [ -z "${entry_json}" ] ; then
    echofatal "${FUNCNAME[0]}() - Internal error: Missing JSON"
    echofatal "Context: ${FUNCNAME[*]}"
    return 100
  fi

  local samba_id=
  samba_id="$( jq -r '.id // empty' <<<"${entry_json}" )" || return $?

  local username=
  username="$( jq -r '.username // empty' <<<"${entry_json}" )" || return $?

  local password=
  password="$( config_json_get_password_by_type_id 'samba-flattern' "${samba_id}" )" || return $?

  local ip=
  ip="$( jq -r '.ip // empty' <<<"${entry_json}" )" || return $?

  local share=
  share="$( jq -r '.["source-share"] // empty' <<<"${entry_json}" )" || return $?

  __mount_samba__ "${username}" "${password}" "${ip}" "${share}" "${samba_id}" || return $?
}

##
## API: build_checkboxes_for_samba
##
function build_checkboxes_for_samba {
  echoinfo "${FUNCNAME[0]}()"

  local ids=
  ids="$( get_all_samba_ids )" || return $?
  local -r ids

  local id=
  while read -r id ; do
    if [ -n "${id}" ] ; then
      __build_checkboxes_for_samba_for_id__ "${id}" || return $?
    fi
  done <<<"${ids}"
}

##
## API: mount_all_samba
##
function mount_all_samba {
  local selected_ids=
  selected_ids="$( get_all_samba_ids )" || return $?
  local -r selected_ids

  local selected_id=
  while read -r selected_id ; do
    if [ -n "${selected_id}" ] ; then
      mount_samba_by_id "${selected_id}" || return $?
    fi
  done <<<"${selected_ids}"
}

##
## API: mount_samba_by_id ID
##
function mount_samba_by_id {
  local -r id="$1"

  local json_entry=
  # shellcheck disable=SC2016 # Expressions don't expand in single quotes, use double quotes for that.
  json_entry="$(
    config_json_get_object \
      --arg 'id' "${id}" \
      '.mount | .["samba-flattern"] // [] | .[] | select( .id == $id )'
    )" || return $?

  __mount_samba_from_json_entry__ "${json_entry}" || return $?
}

##
## API: mount_samba_by_mountdir MOUNT_POINT
##
function mount_samba_by_mountdir {
  local -r mount_point="$1"

  local json_entry=
  # shellcheck disable=SC2016 # Expressions don't expand in single quotes, use double quotes for that.
  json_entry="$(
    config_json_get_object \
      --arg 'mountpoint' "${mount_point}" \
      '.mount | .["samba-flattern"] // [] | .[] | select( .["mount-point"] == $mountpoint )'
    )" || return $?

  __mount_samba_from_json_entry__ "${json_entry}" || return $?
}

##
## API: umount_all_samba
##
function umount_all_samba {
  local mountdirs=
  mountdirs="$( get_all_samba_mountdir )" || return $?
  local -r mountdirs

  local mountdir=
  while read -r mountdir ; do
    if [ -n "${mountdir}" ] ; then
      umount_by_mountdir "${mountdir}" || true # Ignore errors on umount
    fi
  done <<<"${mountdirs}"
}

