#!/bin/bash
#
# mount-manager:bin/api/sshfs-api.sh
#
declare -gr SSHF_MOUNT_DEFAULT_CONFIG=

# Internal
function __build_checkboxes_for_sshfs_for_id__ {
  local -r id="$1"

  echodebug "${FUNCNAME[0]}( $* }"

  local mountdir=
  mountdir="$( get_sshfs_mountdir "${id}" )" || return $?

  update_checkboxes "${mountdir}" "${id}" || return $?
}

# Internal
function __build_sshfs_mount_source__ {
  local -r __user__="$1"
  local -r __host__="$2"
  local -r __directory__="$3"

  #
  # mount.sshfs [user@]host:[dir] mountpoint [options]
  #
  if [ -n "${__user__}" ]; then
    echo -n "${__user__}@"
  fi
  echo -n "${__host__}:"
  if [ -n "${__directory__}" ]; then
    echo -n "${__directory__}"
  fi
}

# Internal
function __mount_sshfs__ {
  local -r __user__="$1"
  local -r __host__="$2"
  local -r __directory__="$3"
  local -r __mount_dir__="$4"
  local -r __sshfs_id__="$5"

  echodebug "${FUNCNAME[0]}( '${__user__}' '${__host__}' '${__directory__}' '${__mount_dir__}' '${__sshfs_id__}' )"

  if [ -z "${__user__}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing '__user__' value."
    return 100
  fi
  if [ -z "${__host__}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing '__host__' value."
    return 100
  fi
  if [ -z "${__directory__}" ] ; then
    echowarn "${FUNCNAME[0]}() '__directory__' is empty."
  fi
  if [ -z "${__mount_dir__}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing '__mount_dir__' value."
    return 100
  fi
  if [ -z "${__sshfs_id__}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing '__sshfs_id__' value."
    return 100
  fi

  local mounted_status=
  mounted_status="$( get_mounted_status "${__mount_dir__}" )" || return $?
  if [ "${mounted_status}" = 'on' ] ; then
    echowarn "Path '${__mount_dir__}' already mounted."
    return 0
  fi

  create_mountdir_if_needed "${__mount_dir__}" || return $?

  local _mount_source_=
  _mount_source_="$( __build_sshfs_mount_source__ "${__user__}" "${__host__}" "${__directory__}" )" || return $?

  local optional=
  optional="$( get_sshfs_optional_options "${__sshfs_id__}" )"
  echodebug "__sshfs_id__ '${__sshfs_id__}' - optional '${optional}'"

  local options="${SSHF_MOUNT_DEFAULT_CONFIG}"
  if [ -n "${optional}" ] && [ "${optional}" != "null" ] ; then
    if [ -n "${options}" ] ; then
      options+=','
    fi
    options+="${optional}"
  fi

  local -i mountrc=0
  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    mount.sshfs -v "${_mount_source_}" "${__mount_dir__}" -o "${options}"
  ) && mountrc=$?

  if [ "${mountrc}" -ne 0 ] ; then
    echowarn "${FUNCNAME[0]}() - Error for id='${sshfs_id}': ${_mount_source_}"
  fi
}

# Internal
function __mount_sshfs_from_json_entry__ {
  local -r entry_json="$1"

  if [ -z "${entry_json}" ] ; then
    echofatal "${FUNCNAME[0]}() - Internal error: Missing JSON"
    echofatal "Context: ${FUNCNAME[*]}"
    return 100
  fi

  local username=
  username="$( jq -r '.username // empty' <<<"${entry_json}" )" || return $?

  local host=
  host="$( jq -r '.host // empty' <<<"${entry_json}" )" || return $?

  local directory=
  directory="$( jq -r '.["source-directory"] // empty' <<<"${entry_json}" )" || return $?

  local mount_point=
  mount_point="$( jq -r '.["mount-point"] // empty' <<<"${entry_json}" )" || return $?

  local sshfs_id=
  sshfs_id="$( jq -r '.id // empty' <<<"${entry_json}" )" || return $?

  __mount_sshfs__ "${username}" "${host}" "${directory}" "${mount_point}" "${sshfs_id}" || return $?
}

##
## API: build_checkboxes_for_sshfs
##
function build_checkboxes_for_sshfs {
  echoinfo "${FUNCNAME[0]}()"

  local ids=
  ids="$( get_all_sshfs_ids )" || return $?
  local -r ids

  local id=
  while read -r id ; do
    if [ -n "${id}" ] ; then
      __build_checkboxes_for_sshfs_for_id__ "${id}" || return $?
    fi
  done <<<"${ids}"
}

##
## API: get_sshfs_mountdir ID
##
function get_sshfs_mountdir {
  local -r id="$1"

  get_mountdir "${id}" || return $?
}

##
## API: mount_all_sshfs
##
function mount_all_sshfs {
  local selected_ids=
  selected_ids="$( get_all_sshfs_ids )" || return $?
  local -r selected_ids

  local selected_id=
  while read -r selected_id ; do
    if [ -n "${selected_id}" ] ; then
      mount_sshfs_by_id "${selected_id}" || return $?
    fi
  done <<<"${selected_ids}"
}

##
## API: mount_sshfs_by_id ID
##
function mount_sshfs_by_id {
  local -r id="$1"

  local json_entry=
  # shellcheck disable=SC2016 # Expressions don't expand in single quotes, use double quotes for that.
  json_entry="$(
    config_json_get_object \
      --arg 'id' "${id}" \
      '.mount | .["sshfs-flattern"] // [] | .[] | select( .id == $id )'
    )" || return $?

  __mount_sshfs_from_json_entry__ "${json_entry}" || return $?
}

##
## API: mount_sshfs_by_mountdir MOUNT_POINT
##
function mount_sshfs_by_mountdir {
  local -r mount_point="$1"

  local json_entry=
  # shellcheck disable=SC2016 # Expressions don't expand in single quotes, use double quotes for that.
  json_entry="$(
    config_json_get_object \
      --arg 'mountpoint' "${mount_point}" \
      '.mount | .["sshfs-flattern"] // [] | .[] | select( .["mount-point"] == $mountpoint )'
    )" || return $?

  __mount_sshfs_from_json_entry__ "${json_entry}" || return $?
}

##
## API: umount_all_sshfs
##
function umount_all_sshfs {
  local mountdirs=
  mountdirs="$( get_all_sshfs_mountdir )" || return $?
  local -r mountdirs

  local mountdir=
  while read -r mountdir ; do
    if [ -n "${mountdir}" ] ; then
      umount_by_mountdir "${mountdir}" || true # Ignore errors on umount
    fi
  done <<<"${mountdirs}"
}

