#!/bin/bash
#
# mount-manager:bin/mount-manager-config.sh (Configuration helper)
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

declare -r MOUNT_MANAGER_PATH="${PROJECT_HOME}/bin"

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${MOUNT_MANAGER_PATH}/api/config-json-api.sh" || exit $?

function usage {
  cat <<EOF
Help to mount and umount samba and nfs shares, or diskimage and iscsi disks.

Usage:

$( basename "$0" ) --help
$( basename "$0" ) [--verbose]

Actions:

  -h,           --help
                        Display help and exit.

Options:

  -v,   --verbose       Show more information, enable debug mode (debugging)
        --debug         Alias for --verbose

Related tools:

  * mount-manager
  * mount-manager-encrypt

EOF
}

function create_config_file {
  local -r config_file="$1"

  if [ -z "${config_file}" ]; then
    echofatal "Config file already exists: '${config_file}'."
    return 100
  fi

  (
    local configdir=
    configdir="$( basename "${config_file}" )" || return $?

    if [ ! -d "${configdir}" ] ; then
      mkdir -vp "${configdir}" || return $?
    fi
  ) || return $?

  touch "${config_file}" || return $?
  chmod 600 "${config_file}" || return $? # Mandatory if plain text password are used (not recommanded)

  jq --sort-keys --null-input '{
  "options":{"@comment":"these values are optionnals","CHECKLIST_HEIGHT":20,"CHECKLIST_WIDTH":80,"CHECKLIST_LISTHEIGHT":10},
  "mount":{"diskimage":[],"iscsi":[],"nfs":[],"samba":[],"sshfs":[]}
}' >>"${config_file}" || return $?
}

function edit_config_file {
  local -r config_file="$1"

  if [ -z "${config_file}" ]; then
    echofatal "Config file is missing."
    return 100
  fi

  if [ -z "${EDITOR:-}" ]; then
    EDITOR=''
  fi

  local -a editor_opts=()
  if [ -z "${EDITOR}" ]; then
    EDITOR="$( command -v xed )"
    if [ -n "${EDITOR}" ] ; then
      editor_opts=(--new-window --wait)
    fi
  fi

  if [ -z "${EDITOR}" ]; then
    EDITOR="$( command -v gedit )"
    if [ -n "${EDITOR}" ] ; then
      editor_opts=(--new-window --wait)
    fi
  fi

  if [ -z "${EDITOR}" ]; then
    EDITOR="$( command -v mousepad )"
  fi

  if [ -z "${EDITOR}" ]; then
    EDITOR='vi'
  fi

  ${EDITOR} "${editor_opts[@]}" "${config_file}"
}

function check_new_configuration {
  echoinfo 'Checking configuration (beta)'
  ( config_json_check_configuration ) || return $?
}

function check_new_config_file {
  local -r config_file="$1"
  local -r config_file_lock="$2"

  if ! check_new_configuration ; then
    whiptail --yesno "New configuration file is corrupt.\n\nRestore previous version ?" 10 50
    local -i result=$?
    echoinfo "whiptail:${result}"

    case "${result}" in
      0)
        echoinfo 'Restore previous version...'
        mv -vf "${config_file_lock}" "${config_file}" || return $?
        ;;

      1)
        echowarn "Keep broken file: '${config_file}' (previous version is '${config_file_lock}')"
        ;;

      *)
        echofatal 'Cancel by user (keep brocken file and previous file)'
        exit 1
        ;;
    esac
  else
    local json=
    json="$( jq -c '.' "${config_file}" )" || return $?
    jq '.' <<<"${json}" >"${config_file}" || return $?
  fi
}

function mount_manager_config {
  local config_file=
  config_file="$( config_json_get_config_file )" || return $?

  local -r config_file_lock="${config_file}.lock"

  if [ ! -f "${config_file}" ]; then
    config_json_try_configuration_file_migration || return $?
  fi
  if [ ! -f "${config_file}" ]; then
    create_config_file "${config_file}" || return $?
  elif [ ! -f "${config_file_lock}" ] ; then
    cp -v "${config_file}" "${config_file_lock}" || return $?
  else
    echofatal 'Lock file exists, editor alreay open ?'
    echofatal "Lock file is there: '${config_file_lock}'"
    echofatal 'Cancel'
    return 123
  fi

  local -i rc=0

  edit_config_file "${config_file}" || rc=$?

  if [ "${rc}" -eq 0 ] ; then
    check_new_config_file "${config_file}" "${config_file_lock}" || rc=$?
  fi
  if [ -f "${config_file_lock}" ] ; then
    rm -v "${config_file_lock}"
  fi

  return "${rc}"
}

function main {
  local param=
  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-v'|'--verbose'|'--debug')
        declare -gx VERBOSE='true'
        declare -gx MESSAGE_API_SHOW_DEBUG='true'
        declare -gx CONFIG_JSON_KEEP_TEMP_FILES='true' # Not use
        echodebug 'Debug mode'
        echodebug 'Verbose mode'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        return 1
        ;;
    esac
  done

  mount_manager_config || return $?

  echoinfo "$( realpath "$0" ) done."
}

main "$@" || exit $?
