#!/bin/bash
#
# mount-manager:bin/mount-manager.sh (Main entry point)
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

declare -r MOUNT_MANAGER_PATH="${PROJECT_HOME}/bin"
declare -r API_PATH="${MOUNT_MANAGER_PATH}/api"

declare -g VERBOSE='false'
declare -grx REALUSER="${USER}"

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/config-json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/crypt-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/diskimage-config-json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/iscsi-config-json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/mount-manager-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/samba-config-json-api.sh" || exit $?
# shellcheck disable=SC1091
source "${API_PATH}/sshfs-config-json-api.sh" || exit $?

function usage {
  cat <<EOF
Help to mount and umount samba and nfs shares, or diskimage and iscsi disks.

Usage:

$( basename "$0" ) --help
$( basename "$0" ) [--verbose]
$( basename "$0" ) --iscsi-logout [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --list [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --list-id [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --mount MOUNTDIR [--mount MOUNTDIR]... [--umount MOUNTDIR]... [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --mount-all [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --mount-default [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --umount MOUNTDIR [--umount MOUNTDIR]... [--mount MOUNTDIR]... [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --umount-all [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --umount-all-force [--verbose] [--no-tests] [--debug]
$( basename "$0" ) --umount-default [--verbose] [--no-tests] [--debug]

Actions:

  -h,           --help
                        Display help and exit.

                --iscsi-logout
                        Logout all iSCSI connections and exit

  -l,           --list  List all defined mount directories.

  -L,           --list-id
                        List all defined id.

  -m MOUNTDIR,  --mount MOUNTDIR
                        Mount MOUNTDIR (if defined in mount-manager configuration)

  -M,           --mount-all
                        Mount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true

  -D,           --mount-default
                        Mount every thing define in configuration file with key:
                            "mount-default": true

  -u MOUNTDIR,  --umount MOUNTDIR
                        Umount MOUNTDIR (if mounted)

  -U,           --umount-all
                        Umount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true

  -F,           --umount-all-force
                        Umount every thing define in configuration file, ignore
                        keys "mount-all" and "mount-default" contents.

  -X,           --umount-default
                        Umount every thing define in configuration file with key:
                            "mount-default": true

Options:

  -v,   --verbose       Show more information (debugging)
        --debug         Enable debug mode
  -Q,   --quick         Alias for --no-tests
        --no-tests      Skip configuration checks

Related tools:

  * mount-manager-config
  * mount-manager-encrypt

EOF
}

function main {
  local    function=
  local    param=
  local    param_all=
  local    param_no_tests=
  local    param_use_cache='true'
  local    param_default=
  local    param_mountdir=
  local    param_verbose=
  local -a param_mountdirs=()
  local -a param_umountdirs=()

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        return 0
        ;;

      '-M'|'--mount-all')
        function='mount_all'
        param_all='true'
        ;;

      '-m'|'--mount')
        function='mount-umount'
        param_mountdir="$( P_param "$@" )" && shift || return $?
        if [ -n "${param_mountdir}" ] ; then
          param_mountdirs+=("${param_mountdir}")
        fi
        ;;

      '-u'|'--umount')
        function='mount-umount'
        param_mountdir="$( P_param "$@" )" && shift || return $?
        if [ -n "${param_mountdir}" ] ; then
          param_umountdirs+=("${param_mountdir}")
        fi
        ;;

      '-D'|'--mount-default')
        function='mount_all'
        param_default='true'
        ;;

      '-U'|'--umount-all')
        function='umount_all'
        param_all='true'
        ;;

      '-F'|'--umount-all-force')
        function='umount_all'
        param_all='false'
        param_default='false'
        ;;

      '-l'|'--list')
        function='list'
        ;;

      '-L'|'--list-id')
        function='list-id'
        ;;

      '-Q'|'--quick'|'--no-tests')
        param_no_tests='true'
        ;;

      '-X'|'--umount-default')
        function='umount_all'
        param_default='true'
        ;;

      '--iscsi-logout')
        function='iscsi_logout'
        ;;

      '-v'|'--verbose')
        param_verbose='true'
        declare -gx MESSAGE_API_SHOW_DEBUG='true'
        echodebug 'Verbose mode'
        ;;

      '--debug')
        param_use_cache='false'
        declare -gx MESSAGE_API_SHOW_DEBUG='true'
        declare -gx CONFIG_JSON_KEEP_TEMP_FILES='true'
        echodebug 'Debug mode'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        return 1
        ;;
    esac
  done

  declare -grx PARAM_ALL="${param_all}"
  declare -grx PARAM_DEFAULT="${param_default}"
  declare -grx VERBOSE="${param_verbose}"

  config_json_init "${param_use_cache}" || return $?
  if [ "${param_no_tests}" != 'true' ] ; then
    config_json_check_configuration || return $?
  fi
  config_json_init_CHECKLIST_values || return $?

  case "${function}" in
    'list')
      list_mountdir || return $?
      ;;

    'list-id')
      list_id || return $?
      ;;

    'umount_all')
      umount_all || true # Ignore errors
      ;;

    'mount_all')
      mount_all || return $?
      ;;

    'mount-umount')
      mountdirs "${param_mountdirs[@]}" || return $?
      umountdirs "${param_umountdirs[@]}" || true # Ignore errors
      ;;

    'iscsi_logout')
      iscsi_logout || true # Ignore errors
      ;;

    *)
      select_mountdirs || return $?
      ;;
  esac
  echookay "$( basename "$0" ) - Done"
}

main "$@" || exit $?

